diff --git a/include/vlc/libvlc_media_player.h b/include/vlc/libvlc_media_player.h
index 8ac784a..a05100f 100644
--- a/include/vlc/libvlc_media_player.h
+++ b/include/vlc/libvlc_media_player.h
@@ -1391,6 +1391,69 @@ VLC_PUBLIC_API int64_t libvlc_audio_get_delay( libvlc_media_player_t *p_mi );
  */
 VLC_PUBLIC_API int libvlc_audio_set_delay( libvlc_media_player_t *p_mi, int64_t i_delay );
 
+/**
+ * Set a random projectm preset
+ *
+ * \param p_mi media player
+ * \version LibVLC 1.1.6 or later
+ */
+VLC_PUBLIC_API void libvlc_projectm_random( libvlc_media_player_t *p_mi );
+
+/**
+ * Get projectm preset count
+ *
+ * \param p_mi media player
+ * \return number of presets loaded by projectm
+ * \version LibVLC 1.1.6 or later
+ */
+VLC_PUBLIC_API  int libvlc_projectm_get_preset_count( libvlc_media_player_t *p_mi );
+
+/**
+ * Get the current preset id
+ *
+ * \param p_mi media player
+ * \return current preset id
+ * \version LibVLC 1.1.6 or later
+ */
+VLC_PUBLIC_API  int libvlc_projectm_get_preset_id( libvlc_media_player_t *p_mi );
+
+/**
+ * Set a projectm preset
+ *
+ * \param p_mi media player
+ * \param i_id preset id
+ * \version LibVLC 1.1.6 or later
+ */
+VLC_PUBLIC_API void libvlc_projectm_set_preset_id( libvlc_media_player_t *p_mi, int i_id );
+
+/**
+ * Get the name of a preset
+ *
+ * \param p_mi media player
+ * \param i_id preset id
+ * \return name of the preset
+ * \version LibVLC 1.1.6 or later
+ */
+VLC_PUBLIC_API char *libvlc_projectm_get_preset_name( libvlc_media_player_t *p_mi, int i_id );
+
+/**
+ * Get whether projectm is locked or not
+ *
+ * \param p_mi media player
+ * \return the lock status (boolean)
+ * \version LibVLC 1.1.6 or later
+ */
+VLC_PUBLIC_API  int libvlc_projectm_is_locked( libvlc_media_player_t *p_mi );
+
+/**
+ * Lock/Unlock the current preset (prevents random switch to new preset)
+ *
+ * \param p_mi media player
+ * \param b_locked boolean for the lock status
+ * \version LibVLC 1.1.6 or later
+ */
+VLC_PUBLIC_API void libvlc_projectm_lock( libvlc_media_player_t *p_mi, int b_locked );
+
 /** @} audio */
 
 /** @} media_player */
diff --git a/modules/visualization/projectm.cpp b/modules/visualization/projectm.cpp
index 948c388..f5f5aab 100644
--- a/modules/visualization/projectm.cpp
+++ b/modules/visualization/projectm.cpp
@@ -119,6 +119,9 @@ struct filter_sys_t
     vout_thread_t  *p_vout;
     vout_display_t *p_vd;
 
+    /* libprojectM objects */
+    projectM      *p_projectm;
+
     /* Window size */
     int i_width;
     int i_height;
@@ -137,6 +140,7 @@ struct filter_sys_t
 
 static block_t *DoWork( filter_t *, block_t * );
 static void *Thread( void * );
+static int ProjectmCallback( vlc_object_t *, char const *, vlc_value_t, vlc_value_t, void * );
 
 /**
  * Open the module
@@ -179,6 +183,23 @@ static int Open( vlc_object_t * p_this )
     p_sys->i_buffer_size = 0;
     p_sys->i_nb_samples = 0;
 
+    var_Create( p_filter, "projectm-preset-id", VLC_VAR_INTEGER );
+    var_Create( p_filter, "projectm-preset-name", VLC_VAR_STRING );
+    var_Create( p_filter, "projectm-cmd-random", VLC_VAR_INTEGER );
+    var_Create( p_filter, "projectm-cmd-get-count", VLC_VAR_INTEGER );
+    var_Create( p_filter, "projectm-cmd-get-preset-id", VLC_VAR_INTEGER );
+    var_Create( p_filter, "projectm-cmd-set-preset-id", VLC_VAR_INTEGER );
+    var_Create( p_filter, "projectm-cmd-get-name", VLC_VAR_INTEGER );
+    var_Create( p_filter, "projectm-cmd-is-locked", VLC_VAR_INTEGER );
+    var_Create( p_filter, "projectm-cmd-lock", VLC_VAR_INTEGER );
+    var_AddCallback( p_filter, "projectm-cmd-random", ProjectmCallback, p_filter );
+    var_AddCallback( p_filter, "projectm-cmd-get-count", ProjectmCallback, p_filter );
+    var_AddCallback( p_filter, "projectm-cmd-get-preset-id", ProjectmCallback, p_filter );
+    var_AddCallback( p_filter, "projectm-cmd-set-preset-id", ProjectmCallback, p_filter );
+    var_AddCallback( p_filter, "projectm-cmd-get-name", ProjectmCallback, p_filter );
+    var_AddCallback( p_filter, "projectm-cmd-is-locked", ProjectmCallback, p_filter );
+    var_AddCallback( p_filter, "projectm-cmd-lock", ProjectmCallback, p_filter );
+
     /* Create the thread */
     if( vlc_clone( &p_sys->thread, Thread, p_filter, VLC_THREAD_PRIORITY_LOW ) )
         goto error;
@@ -217,6 +238,16 @@ static void Close( vlc_object_t *p_this )
 
     vlc_join( p_sys->thread, NULL );
 
+    var_Destroy( p_filter, "projectm-preset-id" );
+    var_Destroy( p_filter, "projectm-preset-name" );
+    var_Destroy( p_filter, "projectm-cmd-random" );
+    var_Destroy( p_filter, "projectm-cmd-get-count" );
+    var_Destroy( p_filter, "projectm-cmd-get-preset-id" );
+    var_Destroy( p_filter, "projectm-cmd-set-preset-id" );
+    var_Destroy( p_filter, "projectm-cmd-get-name" );
+    var_Destroy( p_filter, "projectm-cmd-is-locked" );
+    var_Destroy( p_filter, "projectm-cmd-lock" );
+
     /* Free the ressources */
     vlc_sem_destroy( &p_sys->ready );
     vlc_mutex_destroy( &p_sys->lock );
@@ -272,6 +303,56 @@ static int VoutCallback( vlc_object_t *p_vout, char const *psz_name,
 }
 
 /**
+ * Variable callback for the preset change request
+ */
+static int ProjectmCallback( vlc_object_t *p_vout, char const *psz_name,
+                         vlc_value_t oldv, vlc_value_t newv, void *p_data )
+{
+    filter_t        *p_filter = (filter_t*)p_data;
+    filter_sys_t    *p_sys = p_filter->p_sys;
+
+    vlc_mutex_lock( &p_sys->lock );
+
+    if( !strcmp(psz_name, "projectm-cmd-random") )
+    {
+        p_sys->p_projectm->key_handler( PROJECTM_KEYDOWN, PROJECTM_K_r, PROJECTM_KMOD_LSHIFT );
+    }
+    else if( !strcmp(psz_name, "projectm-cmd-get-count") )
+    {
+        var_SetInteger( p_filter, "projectm-preset-id", (int)p_sys->p_projectm->getPlaylistSize() );
+    }
+    else if( !strcmp(psz_name, "projectm-cmd-get-preset-id") )
+    {
+        unsigned int id;
+        p_sys->p_projectm->selectedPresetIndex( id );
+        var_SetInteger( p_filter, "projectm-preset-id", id );
+    }
+    else if( !strcmp(psz_name, "projectm-cmd-set-preset-id") )
+    {
+        if ( newv.i_int >= 0 && newv.i_int < (int)p_sys->p_projectm->getPlaylistSize() )
+            p_sys->p_projectm->selectPreset( newv.i_int );
+    }
+    else if( !strcmp(psz_name, "projectm-cmd-get-name") )
+    {
+        int id = var_GetInteger( p_filter, "projectm-preset-id" );
+        if ( id >= 0 && id < (int)p_sys->p_projectm->getPlaylistSize() )
+            var_SetString( p_filter, "projectm-preset-name",  p_sys->p_projectm->getPresetName( id ).c_str() );
+    }
+    else if( !strcmp(psz_name, "projectm-cmd-is-locked") )
+    {
+        var_SetInteger( p_filter, "projectm-preset-id", (int)p_sys->p_projectm->isPresetLocked() );
+    }
+    else if( !strcmp(psz_name, "projectm-cmd-lock") )
+    {
+        p_sys->p_projectm->setPresetLock(newv.i_int);
+    }
+
+    vlc_mutex_unlock( &p_sys->lock );
+
+    return VLC_SUCCESS;
+}
+
+/**
  * ProjectM update thread which do the rendering
  * @param p_this: the p_thread object
  */
@@ -287,7 +368,6 @@ static void *Thread( void *p_data )
     locale_t loc;
     locale_t oldloc;
 
-    projectM *p_projectm;
 #ifndef HAVE_PROJECTM2
     char *psz_config;
 #else
@@ -345,7 +425,7 @@ static void *Thread( void *p_data )
     /* Create the projectM object */
 #ifndef HAVE_PROJECTM2
     psz_config = var_InheritString( p_filter, "projectm-config" );
-    p_projectm = new projectM( psz_config );
+    p_sys->p_projectm = new projectM( psz_config );
     free( psz_config );
 #else
     psz_preset_path = var_InheritString( p_filter, "projectm-preset-path" );
@@ -367,21 +447,21 @@ static void *Thread( void *p_data )
     settings.aspectCorrection = 1;
     settings.easterEgg = 1;
     settings.shuffleEnabled = 1;
-    p_projectm = new projectM( settings );
+    p_sys->p_projectm = new projectM( settings );
 
     free( psz_menu_font );
     free( psz_title_font );
     free( psz_preset_path );
 #endif
-    p_sys->i_buffer_size = p_projectm->pcm()->maxsamples;
+    p_sys->i_buffer_size = p_sys->p_projectm->pcm()->maxsamples;
     p_sys->p_buffer = (float*)calloc( p_sys->i_buffer_size,
                                       sizeof( float ) );
 
     vlc_sem_post( &p_sys->ready );
 
     /* Choose a preset randomly or projectM will always show the first one */
-    if ( p_projectm->getPlaylistSize() > 0 )
-        p_projectm->selectPreset( (unsigned)vlc_mrand48() % p_projectm->getPlaylistSize() );
+    if ( p_sys->p_projectm->getPlaylistSize() > 0 )
+        p_sys->p_projectm->selectPreset( (unsigned)vlc_mrand48() % p_sys->p_projectm->getPlaylistSize() );
 
     /* */
     for( ;; )
@@ -395,7 +475,7 @@ static void *Thread( void *p_data )
             /* FIXME it is not perfect as we will have black bands */
             vout_display_place_t place;
             vout_display_PlacePicture( &place, &p_sys->p_vd->source, p_sys->p_vd->cfg, false );
-            p_projectm->projectM_resetGL( place.width, place.height );
+            p_sys->p_projectm->projectM_resetGL( place.width, place.height );
 
             i_last_width  = p_sys->p_vd->cfg->display.width;
             i_last_height = p_sys->p_vd->cfg->display.height;
@@ -405,15 +485,15 @@ static void *Thread( void *p_data )
         vlc_mutex_lock( &p_sys->lock );
         if( p_sys->i_nb_samples > 0 )
         {
-            p_projectm->pcm()->addPCMfloat( p_sys->p_buffer,
-                                            p_sys->i_nb_samples );
+            p_sys->p_projectm->pcm()->addPCMfloat( p_sys->p_buffer,
+                                                   p_sys->i_nb_samples );
             p_sys->i_nb_samples = 0;
         }
         if( p_sys->b_quit )
         {
             vlc_mutex_unlock( &p_sys->lock );
 
-            delete p_projectm;
+            delete p_sys->p_projectm;
             vout_DeleteDisplay( p_sys->p_vd, NULL );
             vlc_object_release( p_sys->p_vout );
             if (loc != (locale_t)0)
@@ -423,9 +503,10 @@ static void *Thread( void *p_data )
             }
             return NULL;
         }
-        vlc_mutex_unlock( &p_sys->lock );
 
-        p_projectm->renderFrame();
+        p_sys->p_projectm->renderFrame();
+
+        vlc_mutex_unlock( &p_sys->lock );
 
         /* */
         mwait( i_deadline );
diff --git a/src/audio_output/input.c b/src/audio_output/input.c
index 8fcd834..402e11d 100644
--- a/src/audio_output/input.c
+++ b/src/audio_output/input.c
@@ -57,6 +57,8 @@ static void inputResamplingStop( aout_input_t *p_input );
 
 static int VisualizationCallback( vlc_object_t *, char const *,
                                   vlc_value_t, vlc_value_t, void * );
+static int ProjectmCallback( vlc_object_t *, char const *,
+                                  vlc_value_t, vlc_value_t, void * );
 static int EqualizerCallback( vlc_object_t *, char const *,
                               vlc_value_t, vlc_value_t, void * );
 static int ReplayGainCallback( vlc_object_t *, char const *,
@@ -373,6 +375,26 @@ int aout_InputNew( aout_instance_t * p_aout, aout_input_t * p_input, const aout_
 
             if( i_visual == 0 ) /* scaletempo */
                 p_input->p_playback_rate_filter = p_filter;
+            else if ( i_visual == 2 && !strcmp(psz_visual, "projectm") && /* projectm */
+                      var_Type( p_aout, "projectm-preset-id" ) == 0 )
+            {
+                var_Create( p_aout, "projectm-preset-id", VLC_VAR_INTEGER );
+                var_Create( p_aout, "projectm-preset-name", VLC_VAR_STRING );
+                var_Create( p_aout, "projectm-cmd-random", VLC_VAR_INTEGER );
+                var_Create( p_aout, "projectm-cmd-get-count", VLC_VAR_INTEGER );
+                var_Create( p_aout, "projectm-cmd-get-preset-id", VLC_VAR_INTEGER );
+                var_Create( p_aout, "projectm-cmd-set-preset-id", VLC_VAR_INTEGER );
+                var_Create( p_aout, "projectm-cmd-get-name", VLC_VAR_INTEGER );
+                var_Create( p_aout, "projectm-cmd-is-locked", VLC_VAR_INTEGER );
+                var_Create( p_aout, "projectm-cmd-lock", VLC_VAR_INTEGER );
+                var_AddCallback( p_aout, "projectm-cmd-random", ProjectmCallback, p_filter );
+                var_AddCallback( p_aout, "projectm-cmd-get-count", ProjectmCallback, p_filter );
+                var_AddCallback( p_aout, "projectm-cmd-get-preset-id", ProjectmCallback, p_filter );
+                var_AddCallback( p_aout, "projectm-cmd-set-preset-id", ProjectmCallback, p_filter );
+                var_AddCallback( p_aout, "projectm-cmd-get-name", ProjectmCallback, p_filter );
+                var_AddCallback( p_aout, "projectm-cmd-is-locked", ProjectmCallback, p_filter );
+                var_AddCallback( p_aout, "projectm-cmd-lock", ProjectmCallback, p_filter );
+            }
 
             /* next filter if any */
             psz_parser = psz_next;
@@ -466,6 +488,16 @@ int aout_InputDelete( aout_instance_t * p_aout, aout_input_t * p_input )
     if ( p_input->b_error )
         return 0;
 
+    var_Destroy( p_aout, "projectm-preset-id" );
+    var_Destroy( p_aout, "projectm-preset-name" );
+    var_Destroy( p_aout, "projectm-cmd-random" );
+    var_Destroy( p_aout, "projectm-cmd-get-count" );
+    var_Destroy( p_aout, "projectm-cmd-get-preset-id" );
+    var_Destroy( p_aout, "projectm-cmd-set-preset-id" );
+    var_Destroy( p_aout, "projectm-cmd-get-name" );
+    var_Destroy( p_aout, "projectm-cmd-is-locked" );
+    var_Destroy( p_aout, "projectm-cmd-lock" );
+
     /* XXX We need to update b_recycle_vout before calling aout_FiltersDestroyPipeline.
      * FIXME They can be a race condition if audio-visual is updated between
      * aout_InputDelete and aout_InputNew.
@@ -791,6 +823,16 @@ static void inputFailure( aout_instance_t * p_aout, aout_input_t * p_input,
     var_Destroy( p_aout, "audio-replay-gain-preamp" );
     var_Destroy( p_aout, "audio-replay-gain-peak-protection" );
 
+    var_Destroy( p_aout, "projectm-preset-id" );
+    var_Destroy( p_aout, "projectm-preset-name" );
+    var_Destroy( p_aout, "projectm-cmd-random" );
+    var_Destroy( p_aout, "projectm-cmd-get-count" );
+    var_Destroy( p_aout, "projectm-cmd-get-preset-id" );
+    var_Destroy( p_aout, "projectm-cmd-set-preset-id" );
+    var_Destroy( p_aout, "projectm-cmd-get-name" );
+    var_Destroy( p_aout, "projectm-cmd-is-locked" );
+    var_Destroy( p_aout, "projectm-cmd-lock" );
+
     /* error flag */
     p_input->b_error = 1;
 }
@@ -895,6 +937,27 @@ static int VisualizationCallback( vlc_object_t *p_this, char const *psz_cmd,
     return VLC_SUCCESS;
 }
 
+static int ProjectmCallback( vlc_object_t *p_this, char const *psz_cmd,
+                             vlc_value_t oldval, vlc_value_t newval, void *p_data )
+{
+    aout_instance_t *p_aout = (aout_instance_t *)p_this;
+    filter_t *p_filter = (filter_t *)p_data;
+    int i_id;
+    char *psz_name;
+
+    i_id = var_GetInteger( p_aout, "projectm-preset-id" );
+    var_SetInteger( p_filter, "projectm-preset-id", i_id );
+
+    var_SetInteger( p_filter, psz_cmd, newval.i_int );
+
+    i_id = var_GetInteger( p_filter, "projectm-preset-id" );
+    psz_name = var_GetString( p_filter, "projectm-preset-name" );
+    var_SetInteger( p_aout, "projectm-preset-id", i_id );
+    var_SetString( p_aout, "projectm-preset-name",  psz_name );
+
+    return VLC_SUCCESS;
+}
+
 static int EqualizerCallback( vlc_object_t *p_this, char const *psz_cmd,
                        vlc_value_t oldval, vlc_value_t newval, void *p_data )
 {
diff --git a/src/control/audio.c b/src/control/audio.c
index a6e7888..d12156b 100644
--- a/src/control/audio.c
+++ b/src/control/audio.c
@@ -515,3 +515,95 @@ int libvlc_audio_set_delay( libvlc_media_player_t *p_mi, int64_t i_delay )
     }
     return ret;
 }
+
+/*****************************************************************************
+ * send_projectm_cmd : Find projectm, send a command, and return the object
+ *****************************************************************************/
+static int send_projectm_cmd( libvlc_media_player_t *p_mi, const char *psz_cmd, int i_value )
+{
+    int id = 0;
+    aout_instance_t *p_aout = GetAOut( p_mi );
+    if( !p_aout )
+        return id;
+
+    if( var_SetInteger( p_aout, psz_cmd, i_value ) < 0 )
+    {
+        libvlc_printerr( "projectm-cmd out of range" );
+        return id;
+    }
+    id = var_GetInteger( p_aout, "projectm-preset-id" );
+    vlc_object_release( p_aout );
+    return id;
+}
+
+
+/*****************************************************************************
+ * libvlc_projectm_random : Set a random projectm preset
+ *****************************************************************************/
+void libvlc_projectm_random( libvlc_media_player_t *p_mi )
+{
+    send_projectm_cmd( p_mi, "projectm-cmd-random", 0 );
+}
+
+/*****************************************************************************
+ * libvlc_projectm_get_preset_count : Get projectm preset count
+ *****************************************************************************/
+int libvlc_projectm_get_preset_count( libvlc_media_player_t *p_mi )
+{
+    return send_projectm_cmd( p_mi, "projectm-cmd-get-count", 0 );
+}
+
+/*****************************************************************************
+ * libvlc_projectm_get_preset_id : Get the current preset id
+ *****************************************************************************/
+int libvlc_projectm_get_preset_id( libvlc_media_player_t *p_mi )
+{
+    return send_projectm_cmd( p_mi, "projectm-cmd-get-preset-id", 0 );
+}
+
+/*****************************************************************************
+ * libvlc_projectm_set_preset : Set a projectm preset
+ *****************************************************************************/
+void libvlc_projectm_set_preset_id( libvlc_media_player_t *p_mi, int i_id )
+{
+    send_projectm_cmd( p_mi, "projectm-cmd-set-preset-id", i_id );
+}
+
+/*****************************************************************************
+ * libvlc_projectm_get_preset_name : Get the name of a preset
+ *****************************************************************************/
+char *libvlc_projectm_get_preset_name( libvlc_media_player_t *p_mi, int i_id )
+{
+    char *name = NULL;
+    aout_instance_t *p_aout = GetAOut( p_mi );
+    if ( !p_aout )
+        return name;
+
+    if( var_SetInteger( p_aout, "projectm-preset-id", i_id ) < 0 )
+    {
+        libvlc_printerr( "projectm-preset-id out of range" );
+    }
+    else
+    {
+        var_SetInteger( p_aout, "projectm-cmd-get-name", 0 );
+        name = var_GetString( p_aout, "projectm-preset-name" );
+    }
+    vlc_object_release( p_aout );
+    return strdup(name);
+}
+
+/*****************************************************************************
+ * libvlc_projectm_is_locked : Get whether projectm is locked or not
+ *****************************************************************************/
+int libvlc_projectm_is_locked( libvlc_media_player_t *p_mi )
+{
+    return send_projectm_cmd( p_mi, "projectm-cmd-is-locked", 0 );
+}
+
+/*****************************************************************************
+ * libvlc_projectm_lock : Lock/Unlock projectm on the current preset
+ *****************************************************************************/
+void libvlc_projectm_lock( libvlc_media_player_t *p_mi, int b_locked )
+{
+    send_projectm_cmd( p_mi, "projectm-cmd-lock", b_locked );
+}
diff --git a/src/libvlc.sym b/src/libvlc.sym
index cdd34ff..3025edf 100644
--- a/src/libvlc.sym
+++ b/src/libvlc.sym
@@ -162,6 +162,13 @@ libvlc_media_subitems
 libvlc_new
 libvlc_new_with_builtins
 libvlc_playlist_play
+libvlc_projectm_get_preset_count
+libvlc_projectm_get_preset_id
+libvlc_projectm_get_preset_name
+libvlc_projectm_set_preset_id
+libvlc_projectm_random
+libvlc_projectm_is_locked
+libvlc_projectm_lock
 libvlc_release
 libvlc_retain
 libvlc_set_fullscreen
